@@ -3,31 +3,38 @@
 #'This function computes per-pair climatologies for the experimental 
 #'and observational data using one of the following methods:
 #'\enumerate{
-#'  \item{per-pair method (Garcia-Serrano and Doblas-Reyes, CD, 2012)}
-#'  \item{Kharin method (Karin et al, GRL, 2012)}
-#'  \item{Fuckar method (Fuckar et al, GRL, 2014)}
+#'  \item{per-pair method (Garcia-Serrano and Doblas-Reyes, CD, 2012 https://doi.org/10.1007/s00382-012-1413-1)}
+#'  \item{Kharin method (Kharin et al, GRL, 2012 https://doi.org/10.1029/2012GL052647)}
+#'  \item{Fuckar method (Fuckar et al, GRL, 2014 https://doi.org/10.1002/2014GL060815)}
 #'}
 #'Per-pair climatology means that only the startdates covered by the 
 #'whole experiments/observational dataset will be used. In other words, the 
 #'startdates which are not all available along 'dat_dim' dimension of both
 #'the 'exp' and 'obs' are excluded when computing the climatologies.
+#'Kharin method is the linear trend bias correction method, and Fuckar method 
+#'is the initial condition bias correction method. The two methods both do the
+#'per-pair correction beforehand.
 #'
-#'@param exp A named numeric array of experimental data, with at least two 
-#'  dimensions 'time_dim' and 'dat_dim'.
-#'@param obs A named numeric array of observational data, same dimensions as  
-#'  parameter 'exp' except along 'dat_dim'.
+#'@param exp A named numeric array of experimental data with at least dimension
+#'  'time_dim'.
+#'@param obs A named numeric array of observational data that has the same 
+#'  dimension as 'exp' except 'dat_dim'.
 #'@param time_dim A character string indicating the name of dimension along  
 #'  which the climatologies are computed. The default value is 'sdate'.
 #'@param dat_dim A character vector indicating the name of the dataset and 
 #'  member dimensions. If data at one startdate (i.e., 'time_dim') are not 
 #'  complete along 'dat_dim', this startdate along 'dat_dim' will be discarded.
-#'  The default value is "c('dataset', 'member')".
-#'@param method  A character string indicating the method to be used. The 
-#'  options include 'clim', 'kharin', and 'NDV'. The default value is 'clim'.
+#'  If there is no dataset dimension, it can be NULL, however, it will be more 
+#'  efficient to simply use mean() to do the calculation. The default value is 
+#'  "c('dataset', 'member')".
+#'@param method A character string indicating the method to be used. The 
+#'  options include 'clim' (per-pair method), 'kharin' (Kharin method), and 
+#'  'NDV' (Fuckar method). The default value is 'clim'.
 #'@param ftime_dim A character string indicating the name of forecast time
 #'  dimension. Only used when method = 'NDV'. The default value is 'ftime'.
 #'@param memb A logical value indicating whether to remain 'memb_dim' dimension
-#'  (TRUE) or do ensemble mean over 'memb_dim' (FALSE). The default value is TRUE.
+#'  (TRUE) or do ensemble mean over 'memb_dim' (FALSE). The default value is 
+#'  TRUE.
 #'@param memb_dim A character string indicating the name of the member 
 #'  dimension. Only used when parameter 'memb' is FALSE. It must be one element
 #'  in 'dat_dim'. The default value is 'member'.
@@ -46,7 +53,7 @@
 #'  dimension 'memb_dim' is also removed.
 #'}
 #'\item{$clim_obs}{
-#'  A numeric array with the same dimensions as  parameter 'exp' 
+#'  A numeric array with the same dimensions as  parameter 'obs' 
 #'  except dimension 'time_dim' is removed. If parameter 'memb' is FALSE,
 #'  dimension 'memb_dim' is also removed.
 #'}
@@ -82,14 +89,10 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     stop(paste0("Parameter 'exp' and 'obs' must be at least two dimensions ",
                 "containing time_dim and dat_dim."))
   }
-  if(any(is.null(names(dim(exp))))| any(nchar(names(dim(exp))) == 0) |
-     any(is.null(names(dim(obs))))| any(nchar(names(dim(obs))) == 0)) {
+  if (any(is.null(names(dim(exp))))| any(nchar(names(dim(exp))) == 0) |
+      any(is.null(names(dim(obs))))| any(nchar(names(dim(obs))) == 0)) {
     stop("Parameter 'exp' and 'obs' must have dimension names.")
   }
-  if(!all(names(dim(exp)) %in% names(dim(obs))) | 
-     !all(names(dim(obs)) %in% names(dim(exp)))) {
-    stop("Parameter 'exp' and 'obs' must have same dimension name")
-  }
   ## time_dim
   if (!is.character(time_dim) | length(time_dim) > 1) {
     stop("Parameter 'time_dim' must be a character string.")
@@ -98,12 +101,26 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     stop("Parameter 'time_dim' is not found in 'exp' or 'obs' dimension.")
   }
   ## dat_dim
-  if (!is.character(dat_dim)) {
-    stop("Parameter 'dat_dim' must be a character vector.")
-  }
-  if (!all(dat_dim %in% names(dim(exp))) | !all(dat_dim %in% names(dim(obs)))) {
-    stop("Parameter 'dat_dim' is not found in 'exp' or 'obs' dimension.")
-  }
+  if (!is.null(dat_dim)) {
+    if (!is.character(dat_dim)) {
+      stop("Parameter 'dat_dim' must be a character vector.")
+    }
+    # Check if dat_dim is in exp
+    if (!all(dat_dim %in% names(dim(exp)))) {
+      stop("Parameter 'dat_dim' is not found in 'exp' dimensions.")
+    }
+    # If dat_dim is not in obs, add it in
+    if (any(!dat_dim %in% names(dim(obs)))) {
+      reset_obs_dim <- TRUE
+      ori_obs_dim <- dim(obs)
+      dim(obs) <- c(dim(obs), rep(1, length(dat_dim[which(!dat_dim %in% names(dim(obs)))])))
+      names(dim(obs)) <- c(names(ori_obs_dim), dat_dim[which(!dat_dim %in% names(dim(obs)))])
+    } else {
+      reset_obs_dim <- FALSE
+    }
+  } else {
+    reset_obs_dim <- FALSE
+  } 
   ## method
   if (!(method %in% c("clim", "kharin", "NDV"))) {
     stop("Parameter 'method' must be one of 'clim', 'kharin' or 'NDV'.")
@@ -122,13 +139,17 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     stop("Parameter 'memb' must be one logical value.")
   }
   ## memb_dim
-  if (!memb) {
+  if (!is.null(memb_dim) & !memb) {
     if (!is.character(memb_dim) | length(memb_dim) > 1) {
       stop("Parameter 'memb_dim' must be a character string.")
     }
     if (!memb_dim %in% names(dim(exp)) | !memb_dim %in% names(dim(obs))) {
-      stop("Parameter 'memb_dim' is not found in 'exp' or 'obs' dimension.")
+      stop("Parameter 'memb_dim' is not found in 'exp' dimension.")
     }
+    if (!memb_dim %in% dat_dim)
+      stop("Parameter 'memb_dim' must be one element in parameter 'dat_dim'.")
+  } else if (is.null(memb_dim) & !memb) {
+    memb <- TRUE
   }
   ## na.rm
   if (!is.logical(na.rm) | length(na.rm) > 1) {
@@ -144,13 +165,15 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
   ## exp and obs (2)
   name_exp <- sort(names(dim(exp)))
   name_obs <- sort(names(dim(obs)))
-  for (i in 1:length(dat_dim)) {
-    name_exp <- name_exp[-which(name_exp == dat_dim[i])]
-    name_obs <- name_obs[-which(name_obs == dat_dim[i])]
+  if (!is.null(dat_dim)) {
+    for (i in 1:length(dat_dim)) {
+      name_exp <- name_exp[-which(name_exp == dat_dim[i])]
+      name_obs <- name_obs[-which(name_obs == dat_dim[i])]
+    }
   }
   if(!all(dim(exp)[name_exp] == dim(obs)[name_obs])) {
-    stop(paste0("Parameter 'exp' and 'obs' must have same length of ",
-                "all dimension expect 'dat_dim'."))
+    stop(paste0("Parameter 'exp' and 'obs' must have the same dimensions ",
+                "expect 'dat_dim'."))
   }
 
   ###############################
@@ -165,8 +188,8 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
   # Calculate Clim
 
   #----------------------------------
-  #  Remove all sdate if not complete along dat_dim
-
+  # Per-pair: Remove all sdate if not complete along dat_dim
+  if (!is.null(dat_dim)) {
     pos <- rep(0, length(dat_dim))
     for (i in 1:length(dat_dim)) {  #[dat, sdate]
       ## dat_dim: [dataset, member]
@@ -182,6 +205,7 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     }
     exp[which(is.na(outrows_exp))] <- NA
     obs[which(is.na(outrows_obs))] <- NA
+  }
 
   #-----------------------------------
 
@@ -193,13 +217,6 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
                   dat_dim = dat_dim, memb_dim = memb_dim,
                   memb = memb, na.rm = na.rm, ncores_input = ncores,
                   ncores = ncores)
-    # Add member dimension name back
-    if (memb) {
-      if(is.null(names(dim(clim$clim_exp))[1])) {
-        names(dim(clim$clim_exp))[1] <- memb_dim
-        names(dim(clim$clim_obs))[1] <- memb_dim
-      }
-    }
 
   } else if (method == 'kharin') {
     clim <- Apply(list(exp, obs),
@@ -218,6 +235,19 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
                   dat_dim = dat_dim, ftime_dim = ftime_dim, memb_dim = memb_dim,
                   memb = memb, na.rm = na.rm, ncores_input = ncores,
                   ncores = ncores)
+  }
+
+  # Remove dat_dim in obs if obs doesn't have at first place
+  if (reset_obs_dim) {
+    clim_obs_dim <- ori_obs_dim[-which(names(ori_obs_dim) == time_dim)]
+    if (!memb & memb_dim %in% names(clim_obs_dim)) {
+      clim_obs_dim <- clim_obs_dim[-which(names(clim_obs_dim) == memb_dim)]
+    }
+    if (is.integer(clim_obs_dim) & length(clim_obs_dim) == 0) {
+     clim$clim_obs <- as.vector(clim$clim_obs)
+    } else {
+      clim$clim_obs <- array(clim$clim_obs, dim = clim_obs_dim)
+    }
   } 
 
   return(clim)
@@ -230,32 +260,43 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
                   na.rm = TRUE, ncores_input = NULL) {
 
   if (method == 'clim') {
-  # exp: [sdate, dat_dim_exp]
-  # obs: [sdate, dat_dim_obs]
+    if (!is.null(dat_dim)) {
+    # exp: [sdate, dat_dim_exp]
+    # obs: [sdate, dat_dim_obs]
 
-    clim_exp <- apply(exp, which(names(dim(exp)) != time_dim),
-                      mean, na.rm = na.rm) #average out time_dim
-    clim_obs <- apply(obs, which(names(dim(obs)) != time_dim),
-                      mean, na.rm = na.rm) #[dat_dim]
-
-  ## member mean
-    if (!memb) {
-      if (length(dim(clim_exp)) == 1) {   #dim: [member]
-        clim_exp <- mean(clim_exp, na.rm = TRUE)    
-        clim_obs <- mean(clim_obs, na.rm = TRUE)
-      } else {
-        pos <- which(names(dim(clim_exp)) == memb_dim)
-        pos <- c(1:length(dim(clim_exp)))[-pos]
-        dim_name <- names(dim(clim_exp))
-        clim_exp <- apply(clim_exp, pos, mean, na.rm = TRUE) 
-        clim_obs <- apply(clim_obs, pos, mean, na.rm = TRUE) 
-        if (is.null(names(dim(as.array(clim_exp))))) {
-          clim_exp <- as.array(clim_exp)
-          clim_obs <- as.array(clim_obs)
-          names(dim(clim_exp)) <- dim_name[pos]
-          names(dim(clim_obs)) <- dim_name[pos]
+      clim_exp <- apply(exp, which(names(dim(exp)) != time_dim),
+                        mean, na.rm = na.rm) #average out time_dim
+      clim_obs <- apply(obs, which(names(dim(obs)) != time_dim),
+                        mean, na.rm = na.rm) #[dat_dim]
+  
+      if (is.null(dim(clim_exp))) {
+        dim(clim_exp) <- length(clim_exp)
+        names(dim(clim_exp)) <- dat_dim
+        dim(clim_obs) <- length(clim_obs)
+        names(dim(clim_obs)) <- dat_dim
+      }
+  
+    ## ensemble mean
+      if (!memb) {
+        if (length(dim(clim_exp)) == 1) {   #dim: [member]
+          clim_exp <- mean(clim_exp, na.rm = TRUE)    
+          clim_obs <- mean(clim_obs, na.rm = TRUE)
+        } else {
+          dim_name <- names(dim(clim_exp))
+          pos <- c(1:length(dim(clim_exp)))[-which(dim_name == memb_dim)]
+          clim_exp <- apply(clim_exp, pos, mean, na.rm = TRUE) 
+          clim_obs <- apply(clim_obs, pos, mean, na.rm = TRUE) 
+          if (is.null(dim(clim_exp))) {
+            dim(clim_exp) <- length(clim_exp)
+            dim(clim_obs) <- length(clim_obs)
+            names(dim(clim_exp)) <- dim_name[pos]
+            names(dim(clim_obs)) <- dim_name[pos]
+          }
         }
       }
+    } else {  #dat_dim = NULL
+      clim_exp <- mean(exp)
+      clim_obs <- mean(obs)
     }
 
   } else if (method == 'kharin') {
@@ -263,27 +304,39 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
   # obs: [sdate, dat_dim_obs]
 
   # obs clim
-    clim_obs <- apply(obs, which(names(dim(obs)) != time_dim), 
-                      mean, na.rm = na.rm) #[dat_dim]
+    if (!is.null(dat_dim)) {
+      clim_obs <- apply(obs, which(names(dim(obs)) != time_dim), 
+                        mean, na.rm = na.rm) #[dat_dim]
+      if (is.null(dim(clim_obs))) {
+        dim(clim_obs) <- length(clim_obs)
+        names(dim(clim_obs)) <- dat_dim
+      }
+    } else {
+      clim_obs <- mean(obs)
+    }
 
   # exp clim
   ##--- NEW trend ---##
-    tmp_obs <- Trend(data = obs, time_dim = time_dim, interval = 1, 
-                     polydeg = 1, conf = FALSE, ncores = ncores_input)$trend
+    tmp_obs <- Trend(data = obs, time_dim = time_dim, interval = 1,
+                     polydeg = 1, conf = FALSE, ncores = ncores_input)$trend 
     tmp_exp <- Trend(data = exp, time_dim = time_dim, interval = 1, 
                      polydeg = 1, conf = FALSE, ncores = ncores_input)$trend
-    # tmp_exp: [stats, dat_dim)]
-
+    # tmp_exp: [stats, dat_dim]
     tmp_obs_mean <- apply(tmp_obs, 1, mean)  #average out dat_dim (dat and member)
     #tmp_obs_mean: [stats = 2]
-
-    intercept_exp <- Subset(tmp_exp, 1, 1, drop = 'selected')  #[dat_dim]
-    slope_exp <- Subset(tmp_exp, 1, 2, drop = 'selected')  #[dat_dim]
-    intercept_obs <- array(tmp_obs_mean[1], dim = dim(exp)[-1])  #[dat_dim]
-    slope_obs <- array(tmp_obs_mean[2], dim = dim(exp)[-1])  #[dat_dim]
+    if (!is.null(dat_dim)) {
+      intercept_exp <- Subset(tmp_exp, 1, 1, drop = 'selected')  #[dat_dim]
+      slope_exp <- Subset(tmp_exp, 1, 2, drop = 'selected')  #[dat_dim]
+      intercept_obs <- array(tmp_obs_mean[1], dim = dim(exp)[-1])  #[dat_dim]
+      slope_obs <- array(tmp_obs_mean[2], dim = dim(exp)[-1])  #[dat_dim]
+    } else {
+      intercept_exp <- tmp_exp[1]
+      slope_exp <- tmp_exp[2]
+      intercept_obs <- tmp_obs_mean[1]
+      slope_obs <- tmp_obs_mean[2]
+    }
     trend_exp <- list()
     trend_obs <- list()
-
     for (jdate in 1:dim(exp)[time_dim]) {
       trend_exp[[jdate]] <- intercept_exp + jdate * slope_exp
       trend_obs[[jdate]] <- intercept_obs + jdate * slope_obs
@@ -291,26 +344,40 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     # turn list into array
     trend_exp <- array(unlist(trend_exp), dim = c(dim(exp)[-1], dim(exp)[1]))
     trend_obs <- array(unlist(trend_obs), dim = c(dim(exp)[-1], dim(exp)[1]))
-    len <- length(dim(exp))
-    trend_exp <- Reorder(trend_exp, c(len, 1:(len - 1)))
-    trend_obs <- Reorder(trend_obs, c(len, 1:(len - 1)))
+    if (!is.null(dat_dim)) {
+      len <- length(dim(exp))
+      trend_exp <- Reorder(trend_exp, c(len, 1:(len - 1)))
+      trend_obs <- Reorder(trend_obs, c(len, 1:(len - 1)))
+    }
 
-    clim_obs_mean <- mean(apply(clim_obs, 1, mean))  #average out dat_dim, get a number
+    # average out dat_dim, get a number
+    if (is.null(dim(clim_obs))) {
+      clim_obs_mean <- mean(clim_obs)
+    } else {
+      clim_obs_mean <- mean(apply(clim_obs, 1, mean))
+    }
     clim_obs_mean <- array(clim_obs_mean, dim = dim(exp)) #enlarge it for the next line
     clim_exp <- trend_exp - trend_obs + clim_obs_mean
-    
-  ## member mean
+
+    ## member mean
     if (!memb) {
-      pos <- which(names(dim(clim_exp)) == memb_dim)
-      pos <- c(1:length(dim(clim_exp)))[-pos]
-      dim_name <- names(dim(clim_exp))
-      clim_exp <- apply(clim_exp, pos, mean, na.rm = TRUE)
-      clim_obs <- apply(clim_obs, pos, mean, na.rm = TRUE)    
-      if (is.null(names(dim(as.array(clim_exp))))) {
+      pos_exp <- c(1:length(dim(clim_exp)))[-which(names(dim(clim_exp)) == memb_dim)]
+      pos_obs <- c(1:length(dim(clim_obs)))[-which(names(dim(clim_obs)) == memb_dim)]
+      tmp_dim_exp <- dim(clim_exp)
+      tmp_dim_obs <- dim(clim_obs)
+      clim_exp <- apply(clim_exp, pos_exp, mean, na.rm = TRUE)
+      if (is.integer(pos_obs) & length(pos_obs) == 0) {
+        clim_obs <- mean(clim_obs)
+      } else {
+        clim_obs <- apply(clim_obs, pos_obs, mean, na.rm = TRUE)
+      }
+      if (is.null(dim(clim_exp))) {
         clim_exp <- as.array(clim_exp)
+        dim(clim_exp) <- tmp_dim_exp[pos_exp]
+      }
+      if (is.null(dim(clim_obs)) & !(is.integer(pos_obs) & length(pos_obs) == 0)) {
         clim_obs <- as.array(clim_obs)
-        names(dim(clim_exp)) <- dim_name[pos]
-        names(dim(clim_obs)) <- dim_name[pos]
+        dim(clim_obs) <- tmp_dim_obs[pos_obs]
       }
     }
 
@@ -319,15 +386,24 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
   # exp: [sdate, dat_dim, ftime]
   # obs: [sdate, dat_dim, ftime]
 
-  # obs clim
+    # obs clim
     clim_obs <- apply(obs, which(names(dim(obs)) != time_dim),
-                      mean, na.rm = na.rm) #[dat_dim, ftime]
+                      mean, na.rm = na.rm) #[(dat_dim), ftime]
 
-  # exp clim
+    if (is.null(dim(clim_obs))) {
+      dim(clim_obs) <- length(clim_obs)
+      names(dim(clim_obs)) <- c(dat_dim, ftime_dim)
+    }
+
+    # exp clim
     pos_ftime <- length(dim(exp))  #a number
     dim_ftime <- dim(exp)[pos_ftime]  #c(ftime = 4)
-    pos_dat <- 2:(length(dim(exp)) - 1)  #1 is sdate, last is ftime
-    dim_dat <- dim(exp)[pos_dat]  #c(dataset = 1, member = 3)
+    if (!is.null(dat_dim)) {
+      pos_dat <- 2:(length(dim(exp)) - 1)  #1 is sdate, last is ftime
+      dim_dat <- dim(exp)[pos_dat]  #c(dataset = 1, member = 3)
+    } else {
+      dim_dat <- NULL
+    }
 
     # Create initial data set (i.e., only first ftime)
     tmp <- Subset(exp, ftime_dim, 1, drop = 'selected')
@@ -350,14 +426,14 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     # Find intercept and slope
     intercept_exp <- Subset(tmp_exp, 1, 1, drop = 'selected')  #[dat_dim, ftime]
     slope_exp <- Subset(tmp_exp, 1, 2, drop = 'selected')  #[dat_dim, ftime]
-    intercept_obs <- array(tmp_obs_mean[1, ], dim = c(dim_ftime, dim_dat))
-    #[ftime, dat_dim] exp
-    intercept_obs <- Reorder(intercept_obs, c(2:length(dim(intercept_obs)), 1))
-    #[dat_dim, ftime] exp
-    slope_obs <- array(tmp_obs_mean[2, ], dim = c(dim_ftime, dim_dat))  
-    #[ftime, dat_dim] exp
-    slope_obs <- Reorder(slope_obs, c(2:length(dim(slope_obs)), 1))
-    #[dat_dim, ftime] exp
+    intercept_obs <- array(tmp_obs_mean[1, ], dim = c(dim_ftime, dim_dat)) #[ftime, dat_dim] exp
+    if (!is.null(dat_dim)) {
+      intercept_obs <- Reorder(intercept_obs, c(2:length(dim(intercept_obs)), 1)) #[dat_dim, ftime] exp
+    } 
+    slope_obs <- array(tmp_obs_mean[2, ], dim = c(dim_ftime, dim_dat)) #[ftime, dat_dim] exp
+    if (!is.null(dat_dim)) {
+      slope_obs <- Reorder(slope_obs, c(2:length(dim(slope_obs)), 1)) #[dat_dim, ftime] exp
+    }
 
     trend_exp <- list()
     trend_obs <- list()
@@ -366,7 +442,9 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
       trend_exp[[jdate]] <- intercept_exp + tmp * slope_exp  #[dat_dim, ftime]
 
       tmp <- array(ini_obs_mean[jdate, ], dim = c(dim_ftime, dim_dat))  #[ftime, dat_dim]
-      tmp <- Reorder(tmp, c(2:length(dim(tmp)), 1))  #[dat_dim, ftime]
+      if (!is.null(dat_dim)) {
+        tmp <- Reorder(tmp, c(2:length(dim(tmp)), 1))  #[dat_dim, ftime]
+      }
       trend_obs[[jdate]] <- intercept_obs + tmp * slope_obs
     }
     # turn list into array
@@ -377,8 +455,11 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
     trend_exp <- Reorder(trend_exp, c(len, 1:(len - 1)))
     trend_obs <- Reorder(trend_obs, c(len, 1:(len - 1)))
     #trend_: [sdate, dat_dim, ftime]
-
+    if (is.null(dim(clim_obs))) {
+      clim_obs_mean <- clim_obs #[ftime]
+    } else {
     clim_obs_mean <- apply(clim_obs, length(dim(clim_obs)), mean)  #average out dat_dim, [ftime]
+    }
     clim_obs_mean <- array(clim_obs_mean, dim = c(dim_ftime, dim(exp)[1], dim_dat))
     #[ftime, sdate, dat_dim]
     len <- length(dim(clim_obs_mean))
@@ -387,18 +468,26 @@ Clim <- function(exp, obs, time_dim = 'sdate', dat_dim = c('dataset', 'member'),
 
     clim_exp <- trend_exp - trend_obs + clim_obs_mean
 
-  ## member mean
+    ## member mean
     if (!memb) {
-      pos <- which(names(dim(clim_exp)) == memb_dim)
-      pos <- c(1:length(dim(clim_exp)))[-pos]
-      dim_name <- names(dim(clim_exp))
-      clim_exp <- apply(clim_exp, pos, mean, na.rm = TRUE)
-      clim_obs <- apply(clim_obs, pos, mean, na.rm = TRUE)
-      if (is.null(names(dim(as.array(clim_exp))))) {
+      pos_exp <- c(1:length(dim(clim_exp)))[-which(names(dim(clim_exp)) == memb_dim)]
+      pos_obs <- c(1:length(dim(clim_obs)))[-which(names(dim(clim_obs)) == memb_dim)]
+
+      tmp_dim_exp <- dim(clim_exp)
+      tmp_dim_obs <- dim(clim_obs)
+      clim_exp <- apply(clim_exp, pos_exp, mean, na.rm = TRUE)
+      if (is.integer(pos_obs) & length(pos_obs) == 0) {
+        clim_obs <- mean(clim_obs)
+      } else {
+        clim_obs <- apply(clim_obs, pos_obs, mean, na.rm = TRUE)
+      }
+      if (is.null(dim(clim_exp))) {
         clim_exp <- as.array(clim_exp)
+        dim(clim_exp) <- tmp_dim_exp[pos_exp]
+      }
+      if (is.null(dim(clim_obs)) & !(is.integer(pos_obs) & length(pos_obs) == 0)) {
         clim_obs <- as.array(clim_obs)
-        names(dim(clim_exp)) <- dim_name[pos]
-        names(dim(clim_obs)) <- dim_name[pos]
+        dim(clim_obs) <- tmp_dim_obs[pos_obs]
       }
     }
 
