@@ -125,10 +125,10 @@ GMST <- function(data_tas, data_tos, data_lats, data_lons, mask_sea_land, sea_va
     stop("The dimension of data_tas and data_tos must be identical.")
   }
   # data_lats and data_lons part1
-  if (!(class(data_lats) == 'numeric' | class(data_lats) == 'integer')) {
+  if (!(inherits(data_lats, 'numeric') | inherits(data_lats, 'integer'))) {
     stop("Parameter 'data_lats' must be a numeric vector.")
   }
-  if (!(class(data_lons) == 'numeric' | class(data_lons) == 'integer')) {
+  if (!(inherits(data_lons, 'numeric') | inherits(data_lons, 'integer'))) {
     stop("Parameter 'data_lons' must be a numeric vector.")
   }
   # lat_dim
